/*
==========================================
Name:		Favorit_01.c
Author:		POSPISIL Frantisek
Created:	12/2008
Chip:		ATmega8535
Frequency:	CPU 8MHz
Hardware:	system mereni a ovladani svetel v automobilu Favorit
==========================================
*/

/* hlavickove soubory */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h> // knihovna kvuli statickym polim
#include "Favorit_01.h"
#include "F_HD44780.h"
#include "F_DS1302.h"
#include "F_DS18B20.h"
#include "F_SetMenu.h"
#include "F_Interrupt.h"

/* definice pole */
static unsigned char Dallas[3][10] PROGMEM = 
{
  { 0x55, 0x28, 0x2C, 0x2C, 0x47, 0x01, 0x00, 0x00, 0xC1, 0xBE },
  { 0x55, 0x28, 0xDE, 0xC9, 0x46, 0x01, 0x00, 0x00, 0x5A, 0xBE },
  { 0x55, 0x28, 0xCC, 0xA7, 0x2F, 0x00, 0x00, 0x00, 0x74, 0xBE }
}; // pole pro adresovani teplomeru

static unsigned char ACGram[48] PROGMEM =
{
  0x00, 0x20, 0x40, 0x7F, 0x80, 0xA0, 0xC0, 0xE0,
  0x10, 0x30, 0x50, 0x7F, 0x90, 0xB0, 0xD0, 0xF0,
  0x18, 0x38, 0x58, 0x7F, 0x98, 0xB8, 0xD8, 0xF8,
  0x1C, 0x3C, 0x5C, 0x7F, 0x9C, 0xBC, 0xDC, 0xFC,
  0x1E, 0x3E, 0x5E, 0x7F, 0x9E, 0xBE, 0xDE, 0xFE,
  0x1F, 0x3F, 0x5F, 0x7F, 0x9F, 0xBF, 0xDF, 0xFF
}; // pole pro kresleni na displej


/* definice funkci */
// ===========================================================================
int main(void)
{
  unsigned char i, Pasek;

  SYS_Init(); // inicializuje system

  while(1)
  {
	LCD_Position( 0, 0 );
	LCD_Print_Timer(); // napise cas z obvodu DS1302
	LCD_WriteSpace( 3 );
	LCD_Print_AD(); // napise napeti z AD prevodniku
	LCD_Position( 1, 0 );
	if((Sterace2&0x80)==0x80) // vypisuje stav steracu
	{
	  LCD_WriteArray( 3 ); // "Cyklus steracu:"
	  LCD_PrDec( ROTcoder + 2 );
	  LCD_WriteData('s');
	  LCD_WriteData(' ');
	  Sterace = ROTcoder;
	}
	else
	{
	  switch(Displej1&0x06) // vypisuje teploty
	  {
	    case 2:
		{
		  LCD_WriteArray( 0 ); // "Venkovni t:"
		  LCD_PrTep( Teplota1 );
		  LCD_WriteSpace( 3 );
		  break;
		}
	    case 4:
		{
		  LCD_WriteArray( 1 ); // "Motor    t:"
		  LCD_PrTep( Teplota2 );
		  LCD_WriteSpace( 3 );
		  break;
		}
	    case 6:
		{
		  LCD_WriteArray( 2 ); // "Vnitrni  t:"
		  LCD_PrTep( Teplota3 );
		  LCD_WriteSpace( 3 );
		  break;
		}
		default: break;
	  }
	}

	if((Otacky1==0xFF) && ((Stav1&0x80)==0x00)) // motor stoji, zapalovani vypnuto
	{
	  LCD_Position( 2, 0 );
	  LCD_WriteArray( 24 ); // "Zapalovani: OFF     "
	  LCD_Position( 3, 0 );
	  if((Stav1&0x0F)==0x00) // odpocitavani pred vypnutim
	  {
	    while((Stav1&0x0F)==0x00)
		{
	      LCD_Position( 0, 0 );
		  LCD_WriteSpace( 20 );
	      LCD_Position( 1, 0 );
	      LCD_WriteArray( 32 ); // "Vypinani: "
		  LCD_PrDec( Timer2-1 );
		  LCD_WriteSpace( 8 );
	      LCD_Position( 2, 0 );
		  LCD_WriteSpace( 20 );
	      LCD_Position( 3, 0 );
		  LCD_WriteArray( 25 ); // "____________________"
		  Wait_ms( 100 );
		}
	  }
	  else LCD_WriteArray( 25 ); // "____________________"
	}
	if((Otacky1==0xFF) && ((Stav1&0x80)==0x80)) // motor stoji, zapalovani zapnuto
	{
	  LCD_Position( 2, 0 );
	  LCD_WriteArray( 26 ); // "Zapalovani: ON      "
	  LCD_Position( 3, 0 );
	  LCD_WriteArray( 27 ); // "++++++++++++++++++++"
	}
	if((Otacky1!=0xFF) && ((Stav1&0x80)==0x00)) // motor bezi, zapalovani vypnuto
	{
	  LCD_Position( 2, 0 );
	  LCD_WriteArray( 28 ); // "Otacky:"
	  LCD_PrDecInt( (unsigned int) (3860000 / Otacky) ); // 3750000
	  LCD_WriteArray( 30 ); // " ot/min "
	  LCD_Position( 3, 0 );
	  LCD_WriteArray( 29 ); // "*! Zapalovani OFF !*"
	}

	if((Otacky1!=0xFF) && ((Stav1&0x80)==0x80)) // motor bezi, zapalovani zapnuto
	{
	  LCD_Position( 2, 0 );
	  LCD_WriteArray( 28 ); // "Otacky:"
	  LCD_PrDecInt( (unsigned int) (3860000 / Otacky) ); // 3750000
	  LCD_WriteArray( 30 ); // " ot/min "
	  LCD_Position( 3, 0 );
	  Pasek = (unsigned char) (62500 / Otacky); // ******************************************************
	  if(Pasek>100) Pasek = 100; // omezeni shora
	  if(Pasek>4) for(i=0;i<(Pasek/5);i++) LCD_WriteData( 0x05 ); // kresli pasek
	  if(Pasek<99) LCD_WriteData( Pasek%5 ); // kresli doplnovany znak
	  if(Pasek<95) for(i=0;i<((95-Pasek)/5);i++) LCD_WriteData( 0x00 ); // kresli mezery
	}


	if((ROT_Key()==0x01) && ((Stav1&0x80)==0x00)) // je stisk koderu a zaroven zastaveno zapalovani
	{
	  ROTcoder = 0;
	  ROTmax =  3;
	  LCD_Init();
	  LCD_WriteComm(0x0F); // blikajici kurzor
	  LCD_WriteArray4( 4 );
	  i = 1;
	  while(i)
	  {
		LCD_Position( ROTcoder, 0 );
		if (ROT_Key())
		{
		  switch(ROTcoder)
		  {
		    case 0: SET_Displej(); break;
			case 1: SET_Cekani(); break;
			case 2: SET_Hodiny(); break;
			case 3: SET_Sterac(); break;
		  }
		  i = 0;
		  LCD_Init();
		}
		Wait_ms( 100 );
		if ((Stav1&0x80)==0x80) 
		{
		  i = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
		  LCD_Init();
		}
	  }
	}
	Wait_ms( 100 );
  }
}

// ===========================================================================
void Teploty(unsigned char val) // cteni z teplomeru
{
  unsigned char i;


  if( ((val&0x0F)<10) && ((val&0xF0)<0x30) )
  {
	i = pgm_read_byte( &Dallas[val>>4][(val&0x0F)] );
	if(i==0x55) DS_Reset();
	DS_Write( i );
  }
  switch(val)
  {
	case 0x0F: 
	{
	  Teplota1 = DS_Read() + ( 0x0100 * DS_Read() );
	  break;
	}
	case 0x1F:
	{
	  Teplota2 = DS_Read() + ( 0x0100 * DS_Read() );
	  break;
	}
	case 0x2F:
	{
	  Teplota3 = DS_Read() + ( 0x0100 * DS_Read() );	
	  break;
	}
	case 0x40:
	{
	  DS_Reset();
	  break;	  
	}  
	case 0x41:
	{
	  DS_Write( 0xCC );
	  break;	  
	}  
	case 0x42:
	{
	  DS_Write( 0x44 );	
	  break;	  
	}  
	default: break;
  }
}

// ===========================================================================
ISR (TIMER1_CAPT_vect) // podprogram preruseni CAPT1 - preteceni casovace 1 spousteno 100x za sekundu
{
  SREG  |= 0x80;		// nastavi globalni preruseni

  ICR1 = 10000;			// perioda preruseni je 10ms = 100x za sekundu
  if(Beep1==0) 			// obsluha BEEP
  {
    TCCR2 = 0b01001000;
	DDRD &= 0b01111111;
  }
  else Beep1--;

  Svetla(); // vyhodnoceni osvetleni

  if((Otacky1==0xF0) && (OtZaloha==Timer1)) 
  {
    Otacky1 = 0xFF; // zastavil se motor
	Teplomer = 0;	// ridi chod mereni pri chodu otackomeru
  }
  if(Otacky1==0xF0) OtZaloha = Timer1; // indikace zastaveni, bude cekat 1s

  if((PINA&0x20)==0x00) // vyhodnoceni steracu
  {
    Sterace1 = 0xFF;
	Clr_Sterace();
	Sterace2 &= 0x7F;
  }
  if((PINA&0x60)==0x20) // vypnuty motor, zapnuty sterac - vypne sterace
  {
    Sterace1 = 0xFF;
	Clr_Sterace();
	Sterace2 &= 0x7F;
	Beep( 3, 20 );
  }
  if((PINA&0x60)==0x60)
  {
    if((Sterace2&0x80)==0x00)
	{
	  Sterace2 |= 0x80;
	  ROTcoder = Sterace;
	  ROTmax =  20;
	}
    if(Sterace1==0xFF)
	{
	  Sterace1 = 0xFE;
	  Sterace2 &= 0x80;
	  Sterace2 |= Timer1;
	  Set_Sterace();
	}
	if((Sterace2&0x7F)==Timer1)
	{
	  Sterace1--;
	  if(Sterace1==0xFC)
	  {
	    Sterace1 = Sterace;
		Clr_Sterace();
	  }
	}
  }

  Timer1++;				// pocita udalosti
  switch(Timer1)		// udalosti ktere se maji stat v intervalu sekund
  {
    case 99: Timer1 = 0; break;
	case 1: // pocita cas osvetleni kabiny
	{
	  if(((Stav1&0x03)==3) || (Stav1==0x10)) Timer2--;
	  break;
	}
	case 2: // pocita cas rizeni svetel kdyz se na neco ceka
	{
	  if(Stav1&0x20) Timer3--;
	  break;
	}
	case 3: // pocita co se ma zobrazovat na druhem radku displeje
	{
	  Displej1++;
	  if(Displej1==7) Displej1 = 1;
	}
	default: break;
  }
  if(Otacky1==0xFF) Teploty(Timer1-5); // kdyz motor stoji, tak se meri
}

// ===========================================================================
void SYS_Init(void) // inicializace procesoru, rozsviti LED
{
  unsigned char i;

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace vystupnich portu
  PORTA = 0b00000000;
  DDRA  = 0b00001010;
  PORTB = 0b00000000;
  DDRB  = 0b10101111;
  PORTC = 0b00000011;
  DDRC  = 0b11111000; 
  PORTD = 0b01001111;
  DDRD  = 0b11110000;

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace casovacu a preruseni
  GICR   = 0b11000000; // povoleni preruseni INT1, INT0
  MCUCR  = 0b00001010; // aktivace preruseni sestupnou hranou  

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace casovace 1
  TCCR1A = 0b11110010; // invertujici rezim obou kanalu, vrchol je ICR1
  TCCR1B = 0b00011010; // kmitocet je 1 000 000 Hz
  ICR1   = 10000;	   // perioda preruseni je 10ms = 100x za sekundu
  OCR1A  = 10050;
  i = DS1302( 0xFF, 0x00 ); // nastaveni jasu LED v procentech
  if(i>100) i = 50;
  OCR1B  = 100 * (100 - i);

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace casovace 0
  TCCR0  = 0b00000011;  // normalni rezim s preddelickou 64
  TIMSK  = 0b00100001; // preruseni pri dosazeni vrcholu ICR1

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace promennych pro casovace
  Otacky1= 0xFF;	   // vyssi bajt citace0
  Timer1 = 0;		   // pocitadlo udalosti v preruseni
  Beep1  = 0;		   // pocita cas pri funkci BEEP
  Stav1  = 0x11;	   // stav osvetleni
  Timer2 = 0;		   // pocitadlo sekund pro osvetleni
  Timer3 = 0;		   // pocitadlo sekund pro osvetleni
  Displej1 = 0;		   // ukazatel teploty zobrazovane na druhem radku
  Sterace2 = 0;		   // prubezny casovac stirani, 7 bit ukazuje 1-stira, 0-nestira
  Teplomer = 0;		   // ridi chod mereni pri chodu otackomeru
  Sterace = DS1302( 0xFD, 0x00 );
  SREG  |= 0x80; 	   // nastavi globalni preruseni

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace A/D prevodniku
  ADMUX  = 0b11000111; // ADC7
  ADCSRA = 0b11110101; // kmitocet 250kHz
  SFIOR  = 0b00000000; // volny beh

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace DS1302
  DS1302_ClrRst();
  i = DS1302( 0b10010000, 0b10100101 ); // aktivace nabijeni
  i = DS1302( 0b10000001, 0b00000000 );
  i = DS1302( 0b10000000, i&0x7F ); // aktivace hodin

  // * * * * * * * * * * * * * * * * * * * * * * * * * Aktivace displeje HD44780
  Wait_ms( 200 );
  LCD_Init();
  Beep( 3, 10 );
  Wait_ms( 150 );
  Beep( 2, 10 );
  LCD_Init();
  for(i=0;i<48;i++) // nahraje data do ACGram
  {
    LCD_WriteComm( i | 0x40 );
	LCD_WriteData( pgm_read_byte( &ACGram[i] ));
  }
  LCD_WriteArray4( 8 ); // vypise hlavicku
  Wait_ms( 1000 );
  LCD_Init();
  ROTmax = 100;
  ROTcoder = 0;
  Teplota1 = 0x00A2; // +10,125
  Teplota2 = 0x00A2; // +10,125
  Teplota3 = 0x00A2; // +10,125
  Otacky = 0x0000;
  OtackyCount = 0; // pocitadlo pri prumerovani
  OtackyERR = 0x00; // pocitadlo chyb pri mereni
}

