/*
==========================================
Name:		F_HD44780.c
Author:		POSPISIL Frantisek
Created:	12/2008
Chip:		ATmega8535
Frequency:	CPU 8MHz
Hardware:	HD44780 komunikace 4 bity
==========================================
*/

/* hlavickove soubory */
#include <avr/io.h>
#include <avr/pgmspace.h> // knihovna kvuli statickym polim
#include "F_HD44780.h"
#include "F_DS1302.h"
#include "F_Texty.h"

/* definice funkci */
// ===========================================================================
void LCD_WriteSpace(unsigned char val ) // napise nejake mezery
{
  unsigned char i;

  for(i=0;i<val;i++) LCD_WriteData( ' ' );
}

// ===========================================================================
void LCD_WriteArray(unsigned char p) // vypisuje text z pole
{
  unsigned char i=0;

  while((pgm_read_byte( &Text[p][i] ))!=0)
  {
    LCD_WriteData( pgm_read_byte( &Text[p][i] ) ); // *************************
	i++;
  }
}

// ===========================================================================
void LCD_Print_AD(void) // napise napeti z AD prevodniku
{
  unsigned int Napeti;

  LCD_WriteData( ' ' );
  LCD_WriteData( 'U' );
  LCD_WriteData( '=' );
  Napeti = (ADCL + (ADCH * 0x100))*2; // precte napeti z prevodniku
  LCD_PrDec( (unsigned char) (Napeti/100) );
  Napeti %= 100;
  LCD_WriteData( ',' );
  LCD_PrDec( (unsigned char) Napeti );
  LCD_WriteData( 'V' );
}

// ===========================================================================
void LCD_Print_Timer(void) // napise cas na displej
{
  LCD_PrHex( DS1302( 0x05, 0x00 )); // napise hodiny
  LCD_WriteData( ':' );
  LCD_PrHex( DS1302( 0x03, 0x00 )); // napise minuty
  LCD_WriteData( ':' );
  LCD_PrHex( DS1302( 0x01, 0x00 )); // napise sekundy
}

// ===========================================================================
void LCD_PrTep(unsigned int val) // vypisuje teplotu se znamenkem
{
  unsigned char i, u;

  if (val&0x0800)
  {
    LCD_WriteData('-');
	val = ~val+1;
  }
  else
  LCD_WriteData('+');
  i = (unsigned char) (val>>4); // cela cast
  val = (val&0x000F) * 625; // desetinna cast
  u = 1;
  if(i>100) LCD_WriteData('1'); // znak 100
  else u = 0;
  i = i % 100;
  if((u==0) && (i/10==0)) u = 1; // je nula, nic se nepise
  else LCD_WriteData((i / 10) + 0x30); // znak 3
  LCD_WriteData((i % 10) + 0x30); // znak 4
  LCD_WriteData(','); // znak 5
  LCD_WriteData((val / 1000) + 0x30); // znak 6
  val = val % 1000;
  LCD_WriteData((val / 100) + 0x30); // znak 7
  LCD_WriteData( 0xDF ); // znak stupen 8
  LCD_WriteData( 'C' );
  LCD_WriteSpace( 3 );
}

// ===========================================================================
void LCD_WriteArray4(unsigned char val)
{
  LCD_Position( 0, 0 );
  LCD_WriteArray( val );
  LCD_Position( 1, 0 );
  LCD_WriteArray( val+1 );
  LCD_Position( 2, 0 );
  LCD_WriteArray( val+2 );
  LCD_Position( 3, 0 );
  LCD_WriteArray( val+3 );
}

// ===========================================================================
void LCD_PrDecInt(unsigned int val) // pet znaku - 16 bitu
{
  unsigned char i, u;
  i = val / 10000;
  if(i==0) LCD_WriteData( ' ' );
  else LCD_WriteData(i + 0x30);
  val = val % 10000;
  u = val / 1000;
  if((i==0) && (u==0)) LCD_WriteData( ' ' );
  else LCD_WriteData(u + 0x30);
  if(u>0) i=u;
  val = val % 1000;
  u = val / 100;
  if((i==0) && (u==0)) LCD_WriteData( ' ' );
  else LCD_WriteData(u + 0x30);
  if(u>0) i=u;
  val = val % 100;
  u = val / 10;
  if((i==0) && (u==0)) LCD_WriteData( ' ' );
  else LCD_WriteData(u + 0x30);
  LCD_WriteData((val % 10) + 0x30);
}

// ===========================================================================
void LCD_PrSDec(unsigned char val) // ctyri znaky - 8 bitu decimalne se znamenkem
{
  if (val&0x80)
  {
    LCD_WriteData('-');
	val = ~val+1;
  }
  else LCD_WriteData(' ');
  LCD_PrDec( val );
}

// ===========================================================================
void LCD_PrHex1(unsigned char val) // jeden znak - spodni 4 bity
{
  val &= 0x0F;
  if (val < 10)
    LCD_WriteData(val+0x30);
  else
    LCD_WriteData(val+0x37);
}

// ===========================================================================
void LCD_PrHexInt(unsigned int val) // ctyri znaky - 16 bitu
{
  unsigned char i;
  i = (unsigned char)(val >> 8);
  LCD_PrHex(i);
  i = (unsigned char)val;
  LCD_PrHex(i);
}

// ===========================================================================
void LCD_PrHex(unsigned char val) // dva znaky - 8 bitu
{
  unsigned char znak;
  znak = val / 16;
  if (znak < 10)
    LCD_WriteData(znak+0x30);
  else
    LCD_WriteData(znak+0x37);
  znak = val % 16;
  if (znak < 10)
    LCD_WriteData(znak+0x30);
  else
    LCD_WriteData(znak+0x37);
}

// ===========================================================================
void LCD_PrDec3(unsigned char val) // tri znaky - 8 bitu decimalne
{
  LCD_WriteData((val / 100) + 0x30);
  LCD_PrDec( val % 100 );
}

// ===========================================================================
void LCD_PrDec(unsigned char val) // dva znaky - 8 bitu decimalne
{
  LCD_WriteData((val / 10) + 0x30);
  LCD_WriteData((val % 10) + 0x30);
}

// ===========================================================================
void LCD_Position(unsigned char y, unsigned char x) // nastavi pozici kurzoru
{
  switch(y)
  {
    case 0: x+=0x00; break;
	case 1: x+=0x40; break;
	case 2: x+=0x14; break;
	case 3: x+=0x54; break;
  }
  LCD_WriteComm(x|0x80);
}

// ===========================================================================
void LCD_Init(void) // provede inicializaci displeje
{
  LCD_ClrEnable();
  LCD_Clear();
  LCD_WriteComm(0x28);
  LCD_WriteComm(0x08);
  LCD_Clear();
  LCD_WriteComm(0x06);
  LCD_WriteComm(0x0C);
}

// ===========================================================================
void LCD_Clear(void) // vymaze obsah displeje a nastavi kurzor na zacatek
{
  LCD_WriteComm(1);
  Wait_ms(100);
}

// ===========================================================================
void LCD_WriteData(unsigned char val) // zapise data na displej
{
  LCD_SetData();
  LCD_Write(val);
}

// ===========================================================================
void LCD_WriteComm(unsigned char val) // zapise ridici kod na displej
{
  LCD_SetCommand();
  LCD_Write(val);
}

// ===========================================================================
void LCD_Write(unsigned char val) // pise na displej
{
  LCD_Data = (LCD_Data & 0xF0) | (val >> 4);
  LCD_SetEnable();
  Wait_us( 20 );
  asm("wdr"); // nuluje watchdog
  LCD_ClrEnable();
  LCD_Data = (LCD_Data & 0xF0) | (val & 0x0F);
  LCD_SetEnable();
  Wait_us( 20 );
  asm("wdr"); // nuluje watchdog
  LCD_ClrEnable();
  Wait_us( 20 );
}

// ===========================================================================
void Wait_ms(unsigned int val) // ceka mikrosekundy
{
  while(val--)
  {
    Wait_us(1000);
  }
}

// ===========================================================================
void Wait_us(unsigned int val) // 8MHz, optimalization Os
{
  while(val--)
  {
    EECR |= 0x00;
  }
}
